
# q-expansion of newform 1840.2.e.b, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1840
weight = 2
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[1151, [1, 0]], [1381, [1, 0]], [737, [-1, 0]], [1201, [1, 0]]]
aps_data = [[0, 0], [0, 2], [2, 1], [0, -1], [0, 0], [0, -2], [0, -5], [8, 0], [0, -1], [5, 0], [5, 0], [0, 7], [-7, 0], [0, 4], [0, 2], [0, 1], [3, 0], [-6, 0], [0, -13], [-13, 0], [0, -8], [-14, 0], [0, -3], [14, 0], [0, 14], [15, 0], [0, 0], [0, -9], [-18, 0], [0, 1], [0, 20], [0, 0], [0, 6], [9, 0], [0, 0], [-8, 0], [0, -3], [0, 24], [0, -16], [0, -6], [-4, 0], [-14, 0], [8, 0], [0, 12], [0, 0], [2, 0], [9, 0], [0, -14], [0, 0], [-4, 0], [0, -6], [-1, 0], [6, 0], [-18, 0], [0, 22], [0, -13], [15, 0], [15, 0], [0, -26], [-12, 0], [0, 11], [0, -29], [0, -14], [-4, 0], [0, -21], [0, -24], [5, 0], [0, 26], [0, -8], [-23, 0], [0, 0], [-6, 0], [0, 13], [0, 6], [6, 0], [0, 3], [-22, 0], [0, 16], [2, 0], [19, 0], [-2, 0], [10, 0], [12, 0], [0, -19], [-28, 0], [0, -24], [-15, 0], [0, 1], [-18, 0], [0, -4], [0, 27], [-28, 0], [0, 32], [31, 0], [-11, 0], [0, -9], [-26, 0], [-20, 0], [0, -28], [14, 0], [0, 4], [0, -33], [0, -25], [12, 0], [-18, 0], [0, 22], [0, 12], [0, -30], [-48, 0], [-25, 0], [0, 38], [0, -6], [0, -47], [-10, 0], [20, 0], [-24, 0], [0, 37], [0, 18], [0, -2], [22, 0], [-16, 0], [0, -24], [0, -3], [0, 46], [28, 0], [-28, 0], [-4, 0], [-45, 0], [0, -27], [0, -7], [7, 0], [0, -32], [-14, 0], [0, -35], [25, 0], [-10, 0], [0, 42], [0, -17], [0, -15], [25, 0], [-31, 0], [30, 0], [0, -34], [0, 9], [-37, 0], [14, 0], [0, -8], [0, 24], [43, 0], [0, -18], [0, 8], [54, 0], [0, -20], [0, -42], [0, -9], [12, 0], [4, 0], [-35, 0], [0, 54], [2, 0], [0, -14], [0, -54], [0, 18], [6, 0], [0, -39], [0, 21], [-19, 0], [0, 20], [6, 0], [0, 8], [52, 0], [41, 0], [-44, 0], [0, 61], [-48, 0], [-50, 0], [16, 0], [11, 0], [0, 35], [64, 0], [0, 36], [-24, 0], [0, 30], [0, 42], [0, -9], [-8, 0], [0, 16], [0, -37], [-49, 0], [-37, 0], [0, 64], [0, 26], [38, 0], [-3, 0], [0, -12], [0, 27], [12, 0], [0, -46], [0, -62], [0, 24], [-12, 0], [40, 0], [0, 26], [-8, 0], [42, 0], [0, 48], [14, 0], [0, -60], [-29, 0], [-28, 0], [0, -20], [62, 0], [0, -28], [0, -12], [-37, 0], [-64, 0], [0, 14], [-51, 0], [0, -39], [0, 64], [-31, 0], [44, 0], [-3, 0], [0, 15], [0, -40], [1, 0], [0, 6], [-33, 0], [0, 0], [-17, 0], [0, -2], [2, 0], [-66, 0], [-57, 0], [0, -31], [0, 16], [-6, 0], [0, -10], [75, 0], [3, 0], [0, 21], [-27, 0], [0, -28], [26, 0], [0, -56], [0, 0], [0, -46], [-46, 0], [-40, 0], [0, 41], [0, 30], [50, 0], [0, 17], [-6, 0], [0, -14], [-21, 0], [38, 0], [0, 52], [0, -24], [0, 44], [0, 79], [0, -75], [-7, 0], [0, -13], [0, -70], [0, 0], [70, 0], [8, 0], [0, -29], [0, 62], [-63, 0], [0, -17], [0, -51], [-44, 0], [0, 48], [0, -34], [0, -18], [11, 0], [42, 0], [-26, 0], [0, 36], [-38, 0], [0, -72], [-16, 0], [0, 8], [-45, 0], [0, -63], [0, 38], [-32, 0], [-30, 0], [38, 0], [0, 53], [0, 22], [12, 0], [0, -58], [34, 0], [14, 0], [0, 20], [-35, 0], [0, -10], [0, -17], [0, 59], [44, 0]]
